#include "ell/SC_Listing.hh"

namespace ell {
	
	SC_Listing::SC_Listing()
	  :	SC_Counting(), stateList() 
	{}
	
	SC_Listing::~SC_Listing()
	{
		  // delete all collected State objects
		for (	List::iterator it = stateList.begin();
				it != stateList.end(); it++ ) 
		{
			delete *it;
		}
		  // clear NULL pointer list
		stateList.clear();
	}

	  // This function is used to track all added intermediate States.
	  // @param s the added State
	void 
	SC_Listing::add(const State& s) {
		  // store a copy of s
		stateList.push_back(s.clone());
	}
	
	  // Returns number of added States.
	  // @return the number of added states
	size_t 
	SC_Listing::size() const {
		return stateList.size();
	}
	 
	  // Returns last added State.
	  // @return the last added State
	const State* const 
	SC_Listing::getLastAdded() const {
		return *(stateList.rbegin());
	}
	
	///////////  LIST HANDLING  ////////////////////////

	  // Returns a list of all added States.
	  // @return the State list
	const SC_Listing::List&
	SC_Listing::getList() const {
		return stateList;
	}
	

} // namespace ell
