#include "ell/StateAcceptor.hh"
#include "biu/RandomNumberFactory.hh" // for SA_Metropolis
#include <math.h> // exp for SA_Metropolis


using namespace ell;

bool
SA_All::accept(const StateCollector* const sc, const State& succ) const
{
	return true;
}

bool
SA_E_Eq::accept(const StateCollector* const sc, const State& succ) const
{
	if ( succ.getEnergy() == sc->getLastAdded()->getEnergy())
		return true;
	else
		return false;
}

bool
SA_E_Gr::accept(const StateCollector* const sc, const State& succ) const
{
	if ( succ.getEnergy() > sc->getLastAdded()->getEnergy())
		return true;
	else
		return false;
}

bool
SA_E_GrEq::accept(const StateCollector* const sc, const State& succ) const
{
	if ( succ.getEnergy() >= sc->getLastAdded()->getEnergy())
		return true;
	else
		return false;
}

bool
SA_E_Le::accept(const StateCollector* const sc, const State& succ) const
{
	if ( succ.getEnergy() < sc->getLastAdded()->getEnergy())
		return true;
	else
		return false;
}

bool
SA_E_LeEq::accept(const StateCollector* const sc, const State& succ) const
{
	if ( succ.getEnergy() <= sc->getLastAdded()->getEnergy())
		return true;
	else
		return false;
}

bool
SA_EC_Le::accept(const StateCollector* const sc, const State& succ) const
{
	if ( succ.getEnergy() < sc->getLastAdded()->getEnergy()) {
		return true;
	} else if ( succ.getEnergy() == sc->getLastAdded()->getEnergy()
				&& succ.compress() < sc->getLastAdded()->compress() ) 
	{
		return true;
	} else {
		return false;
	}
}

SA_Metropolis::SA_Metropolis(const double _beta) :beta(_beta) {}

bool
SA_Metropolis::accept(const StateCollector* const sc, const State& succ) const
{	
	// compute energy difference
	double deltaE = succ.getEnergy() - sc->getLastAdded()->getEnergy();
	// compute probability for accepting state -> Metropolis
	double p = exp((-1)*beta*deltaE);
	
	// compute random number 0 <= r <= 1
	double r = (((double) biu::RNF::getRN()) / ((double) biu::RNF::getMaxRN())); 

	// return true with probability p
	if (p>r) return true;
	else return false;
}
