#ifndef DUMMYSTATE_HH_
#define DUMMYSTATE_HH_

using namespace ell;

#include <sstream>

/*!
 * This is a dummy implementation of a State class including inner classes
 * NeighborList and ItState.
 * 
 * @author Daniel Maticzka
 */
class DummyState : public State
{
public:
	class NeighborList : public State::NeighborList 
	{
	private:
		std::string id;
		double energy;
	public:
		class ItState : public State::NeighborList::ItState {
		public:
			int pos;
			
			ItState():pos(0) {}
			
			ItState(const ItState &itstate) {pos=itstate.pos;}
			
			virtual ItState * clone() const {
				return new ItState(*this);
			}
		};
    
		NeighborList(std::string _id, double _energy) 
		{
			id = _id;
			energy = _energy;
		}
    
		virtual ~NeighborList() {}
  
		virtual State*
		next(State::NeighborList::ItState *itstate, State* elem) const {
			((DummyState*)elem)->applyMove();
			return elem;
		}

		virtual State* 
		first(State::NeighborList::ItState** itstate) const {
			DummyState* s = new DummyState(id, energy);
			s->applyMove();
			return s;
		}
	};
	
	std::string id;
	double energy;

	DummyState(std::string _id, double _energy) { id = _id; energy = _energy; } 
	~DummyState() {}
	
	bool operator== (const State& state2) const { return toString() == state2.toString(); }
	bool operator!= (const State& state2) const { return toString() != state2.toString(); }

	int	getFitness() const { return 0; }
	double getEnergy() const { return energy; }
	unsigned int getMinimalDistance(const State& state2) const { return 0; }
	State* clone() const { return new DummyState(id, energy); }
	State* fromString(const std::string& stringRep) const {
		 // get energy
		size_t delim = stringRep.rfind("#");
		if (delim == stringRep.size() || delim == stringRep.size()-1)
			return NULL;
		std::stringstream ss(stringRep.substr(delim+1));
		int e = INT_MAX;
		ss >>e;
		  // create new object
		State* tmp =  new DummyState(stringRep.substr(0,delim), e);
		return tmp;
	}
	std::string	toString() const {
		std::stringstream ss;
		ss <<id <<"#"<<energy;
		return ss.str(); 
	}
	NeighborListPtr getNeighborList() const 
	{ 
		return NeighborListPtr(new NeighborList(id,energy));
	}
	NeighborListPtr getRandomNeighborList() const {
		return NeighborListPtr(new NeighborList(id,energy));
	}
	State* getRandomNeighbor(State* inPlaceNeigh = NULL) const { return NULL; }
	DummyState& applyMove() {
		energy -= 1;
		return *this;
	}
	//////////////  COMPRESSED REPRESENTATION  ////////////////////////////
	
	CSequence  compress(void) const {
		return CSequence();
	}
	
	CSequence&   compress(CSequence& toFill) const {
		toFill.clear();
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq, State* toFill) const {
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq) {
		return NULL;
	}

};

#endif /*DUMMYSTATE_HH_*/
