#include "ell/StateCollector.hh"
#include "ell/State.hh"
#include <iostream>
#include <iterator>
#include <vector>
#include <list>

#include "ell/SC_Counting.hh"
#include "ell/SC_Outstream.hh"
#include "ell/SC_Listing.hh"
#include "ell/SC_ListingCompr.hh"

using namespace ell;

/*!
 * This is a dummy implementation of a State class. The only functionality
 * implemented is setting an id in the constructor and retrieving it via
 * toString.
 * 
 * @author Daniel Maticzka
 */
class DummyState : public State
{
	public:
	
	std::string id;
	
	static const biu::Alphabet a;

	DummyState(std::string _id)
	 :	id(_id)
	{
		id = _id; 
	}
	
	~DummyState() {}
	
	bool operator== (const State& state2) const {return true;}
	bool operator!= (const State& state2) const {return true;}

	int	getFitness() const 
	{
		return 0;
	}
	
	double getEnergy() const 
	{
		return 0;
	}
	
	unsigned int getMinimalDistance(const State& state2) const 
	{
		return 0;
	}
	
	State* clone() const
	{
		return new DummyState(id);
	}
	
	State* fromString(const std::string& stringRep) const { return NULL; }
	
	std::string	toString() const 
	{
		return id;
	}

	NeighborListPtr getNeighborList() const
	{
		return *(new NeighborListPtr);
	}

	NeighborListPtr getRandomNeighborList() const
	{
		return *(new NeighborListPtr);
	}

	State* getRandomNeighbor(State* inPlaceNeigh = NULL) const
	{
		return NULL;
	}

	//////////////  COMPRESSED REPRESENTATION  ////////////////////////////
	
	CSequence  compress(void) const {
		return a.compress(a.getSequence(id));
	}
	
	CSequence&   compress(CSequence& toFill) const {
		toFill.clear();
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq, State* toFill) const {
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq) {
		return NULL;
	}

};

const biu::Alphabet DummyState::a = biu::Alphabet("State 12345",1);

void fillSC( StateCollector& sc, const std::list<State*>& l ) {
	std::list<State*>::const_iterator it = l.begin();
	std::cout << "->add( State 2 - 4 )" << std::endl;
	for (it++; it != l.end(); it++)
		sc.add(**it);
}

void printSCstat(const StateCollector& sc, std::ostream &out) {
	
	out	<<"->getlastAdded()->toString() = " 
		<<sc.getLastAdded()->toString()
		<<std::endl; 
	out	<<"->size() = " 
		<<sc.size()
		<<std::endl; 
}

int main(int argc, char** argv)
{
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test interface : StateCollector\n"
				<<"===========================================" <<std::endl;
	
	// create some States
	std::cout 	<< "=> creating 5 DummyStates";
	std::list<State*> l;
	l.push_back(new DummyState("State 1"));
	l.push_back(new DummyState("State 2"));
	l.push_back(new DummyState("State 3"));
	l.push_back(new DummyState("State 4"));
	l.push_back(new DummyState("State 5"));
	std::list<State*>::const_iterator it;
	
	
	{
		std::cout	<<std::endl
					<<"===========================================\n"
					<<" test class : SC_Counting\n"
					<<"===========================================" <<std::endl;
		  // create new object	
		std::cout 	<< "=> creating ell::SC_Counting" << std::endl;
		SC_Counting sc = SC_Counting();
		  // fill SC
		fillSC(sc, l);
		  // get SC stats
		printSCstat(sc,std::cout);
	}

	{
		std::cout	<<std::endl
					<<"===========================================\n"
					<<" test class : SC_Outstream\n"
					<<"===========================================" <<std::endl;
		  // create new object	
		std::cout 	<< "=> creating ell::SC_Outstream" << std::endl;
		SC_Outstream sc = SC_Outstream(std::cout, " accepted");
		  // fill SC
		fillSC(sc, l);
		  // get SC stats
		printSCstat(sc,std::cout);
	}

	{
		std::cout	<<std::endl
					<<"===========================================\n"
					<<" test class : SC_Listing\n"
					<<"===========================================" <<std::endl;
		  // create new object	
		std::cout 	<< "=> creating ell::SC_Listing" << std::endl;
		SC_Listing sc = SC_Listing();
		  // fill SC
		fillSC(sc, l);
		  // get SC stats
		printSCstat(sc,std::cout);
		std::cout <<"=>stored list :" <<std::endl;
		SC_Listing::List sl = sc.getList();
		for( SC_Listing::List::const_iterator x = sl.begin(); x!=sl.end(); x++)
			std::cout <<"  " <<(*x)->toString() <<std::endl;
	}

	{
		std::cout	<<std::endl
					<<"===========================================\n"
					<<" test class : SC_ListingCompr\n"
					<<"===========================================" <<std::endl;
		  // create new object	
		std::cout 	<< "=> creating ell::SC_ListingCompr" << std::endl;
		SC_ListingCompr sc = SC_ListingCompr();
		  // fill SC
		fillSC(sc, l);
		  // get SC stats
		printSCstat(sc,std::cout);
		std::cout <<"=>stored uncompressed list :" <<std::endl;
		SC_ListingCompr::List sl = sc.getList();
		for( SC_ListingCompr::List::const_iterator x = sl.begin(); x!=sl.end(); x++) {
			std::cout <<"  " << DummyState::a.getString(DummyState::a.decompress(*x,7)) <<std::endl;
		}
	}

	  // clear memory of dummy states
	for (it = l.begin(); it != l.end(); it++)
		delete (*it);

	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	return 0;
}
